/***************************************************************************
****************************************************************************
****************************************************************************
* iptrk - ip tracker
* Copyright (C) 2003 Jason Nunn <jsno@leal.com.au>
* Darwin, NT, Australia.
* Originally developed for Arafura Connect (Dec 2000 - Mar 2003)
* Donated to GPL before liquidation.
*
* =======================================================================
*
****************************************************************************
****************************************************************************
***************************************************************************/
#include "defs.h"
#include "lib.p"

/***************************************************************************
*
***************************************************************************/
PUBLIC void ll_append_unsorted(ll_t **n, void *entry, int entry_size)
{
  ll_t *tmp;

  tmp = *n;
  *n = alloc_mem(sizeof(ll_t) + entry_size);
  (*n)->next = tmp;
  memcpy((void *)LL_ENTRY(*n), entry, entry_size);
}

/***************************************************************************
*
***************************************************************************/
PUBLIC void *ll_find(ll_t **ll, int (*cmp_func)(ll_t *))
{
  ll_t *n;
  void *e;

  n = *ll;
  for(;;)
  {
    if(n == NULL)
      return NULL;
    e = LL_ENTRY(n);
    if(cmp_func(e))
      return e;
    n = n->next;
  }
  return NULL;
}

/***************************************************************************
*
***************************************************************************/
PUBLIC void ll_remove_entry(ll_t **n)
{
  ll_t *tmp;

  if(*n == NULL)
    return;
  tmp = (*n)->next;
  free(*n);
  *n = tmp;
}

/***************************************************************************
*
***************************************************************************/
PUBLIC int ll_remove(ll_t **ll, int (*cmp_func)(ll_t *))
{
  ll_t **n;

  n = ll;
  for(;;)
  {
    if(*n == NULL)
      break;

    if(cmp_func(LL_ENTRY(n)))
    {
      ll_remove_entry(n);
      return 1;
    }

    n = &((*n)->next);
  }

  return 0;
}

/***************************************************************************
*
***************************************************************************/
PUBLIC void ll_free(ll_t **ll)
{
  ll_t **n, *tmp;

  n = ll;
  for(;;)
  {
    if(*n == NULL)
      break;
    tmp = (*n)->next;
    free(*n);
    *n = tmp;
  }
  *ll = NULL;
}

/***************************************************************************
*
***************************************************************************/
PUBLIC void *ll_read_entry(ll_t **ll_ptr)
{
  void *e;

  if(*ll_ptr == NULL)
    return NULL;
  e = (void *)LL_ENTRY(*ll_ptr);
  *ll_ptr = (*ll_ptr)->next;
  return e;
}
